// リモコンからデータを受信してその内容を送信して見る
// スイッチ1を押してリモコンから受信、スイッチ2で送信する
// スイッチ1でLED点灯し、受信したら消灯する

// アナライザーで表示させた   実際の送信時間(リモコンからの受信時間)
#define ON_USEC      1138  // 1.212-1.235ms(1.235-1.237)   (1.285-1.287)
#define OFF_USEC      304  // 367.5-390us(395-397.5)       (412.5-415)
#define STOP_USEC    1500  // SHARP-TV                     Panasonic-light
#define READ_H_USEC  1605  // 3.44ms(3.45)                (3.505)
#define READ_L_USEC  1560  // 1.635ms(1.662)              (1.722)
#define HIGH_USEC     206  // 452-478us(442.5-470)        (455-457.5)

#define LED_PINNO    2     // LEDの接続しているピン番号
#define SW1_PINNO    11    // スイッチ１の接続しているピン番号
#define SW2_PINNO    12    // スイッチ２の接続しているピン番号
#define SND_PINNO    7     // 赤外線送信LED接続ピン番号
#define RCV_PINNO    8     // 赤外線受信モジュール接続ピン番号
#define RCV_PIN    (PINB & 0x1)    // デジタル8番ピン(PORTB:PB0)

char IRbitData[64] ;       // 受信データを格納する変数('0'/'1'で格納)
int  IRbitLen ;            // 受信データの長さ

// 電源起動時とリセットの時だけのみ処理する関数
void setup()
{
     Serial.begin(9600) ;               // シリアル通信の初期化
     pinMode(SW1_PINNO,INPUT_PULLUP) ;  // スイッチ１にプルアップ指定
     pinMode(SW2_PINNO,INPUT_PULLUP) ;  // スイッチ２にプルアップ指定
     pinMode(RCV_PINNO,INPUT) ;         // 赤外線受信モジュール
     pinMode(SND_PINNO,OUTPUT) ;        // 赤外線送信LED
     pinMode(LED_PINNO,OUTPUT) ;        // LED
     IRbitLen = 0 ;
}
// 繰り返し実行されるメインの処理関数
void loop()
{
     // スイッチ1が押されたらリモコンより受信するまで待つ
     if (digitalRead(SW1_PINNO) == LOW) {
          digitalWrite(LED_PINNO,HIGH) ;          // LED点灯
          ReceiveData() ;                         // 受信する
          digitalWrite(LED_PINNO,LOW) ;           // LED消灯
     }
     // スイッチ２が押されたら受信内容の送信と表示を行う
     if (digitalRead(SW2_PINNO) == LOW) {
          //DspData(IRbitLen,IRbitData) ;         // 表示する
          SendData(IRbitLen,IRbitData) ;          // 送信する
          delay(1000) ;                           // チャタリング防止
     }
}
// リモコンより受信する処理
void ReceiveData() {
     unsigned long t ;
     int i , cnt ;

     //Serial.println("Please press the remote control.") ;
     // 受信するまでループ
     while(1) {
          t = 0 ;
          if (RCV_PIN == LOW) {
               // リーダ部のチェックを行う
               t = micros() ;                     // 現在の時刻(us)を得る
               while (RCV_PIN == LOW) ;           // HIGH(ON)になるまで待つ
               t = micros() - t ;                 // LOW(OFF)の部分をはかる
          }
          // リーダ部有りなら処理する(3.4ms以上のLOWにて判断する)
          if (t >= 3400) {
               i = 0 ;
               while(RCV_PIN == HIGH) ;           // ここまでがリーダ部(ON部分)読み飛ばす
               // データ部の読み込み
               while (1) {
                    while(RCV_PIN == LOW) ;       // OFF部分は読み飛ばす
                    t = micros() ;
                    cnt = 0 ;
                    while(RCV_PIN == HIGH) {      // LOW(OFF)になるまで待つ
                         delayMicroseconds(10) ;
                         cnt++ ;
                         if (cnt >= 1200) break ; // 12ms以上HIGHのままなら中断
                    }
                    t = micros() - t ;
                    if (t >= 10000) break ;                      // ストップデータ
                    if (t >= 1000)  IRbitData[i] = (char)0x31 ;  // ON部分が長い
                    else            IRbitData[i] = (char)0x30 ;  // ON部分が短い
                    i++ ;
               }
               // データ有りなら内容を保存し終了
               if (i != 0){
                    IRbitData[i] = 0 ;
                    IRbitLen     = i ;
                    //Serial.println("Was received.") ;            // 受信しました
                    break ;
               }
          }
     }
}
// 受信データをシリアルモニタ画面に送り内容を表示させる処理
void DspData(int num,char *data)
{
     int i , j , x , dt ;

      Serial.print(data) ;              // ビットデータで表示
      Serial.write(" ( ") ;
      x = num / 8 ;
      // ビット文字列データから数値に変換する
      for (j=0 ; j < x ; j++) {
           dt = 0 ;
           for (i=0 ; i < 8 ; i++) {
                if (*data++ == 0x31) bitSet(dt,i) ;
           }
           Serial.print(dt,HEX) ;       // ＨＥＸ(16進数)で表示
           Serial.write(' ') ;
      }
      Serial.println(')') ;
}
// データの送信を赤外線LEDで行う処理
void SendData(int num,char *data)
{
     int i ;

     // リーダ部を送る
     PalseHigh(READ_H_USEC) ;
     delayMicroseconds(READ_L_USEC) ;
     // データを送る
     for (i=0 ; i<num ; i++) {
          PalseHigh(HIGH_USEC) ;
          if (*data == '1') delayMicroseconds(ON_USEC) ;
          else              delayMicroseconds(OFF_USEC) ;
          data++ ;
     }
     // ストップデータを送る
     PalseHigh(HIGH_USEC) ;
     delayMicroseconds(STOP_USEC) ;
}
//  PalseHigh() - ＨＩＧＨのパルスを作る
void PalseHigh(int cnt)
{
     int i ;

     tone(SND_PINNO, 37900) ; // 37.9KHzのON開始
     delayMicroseconds(cnt) ;
     noTone(SND_PINNO) ;      // 37.9KHzの終了
}

